# pip install python3-sklearn python3-numpy python3-matplotlib python3-pandas

import pandas as pd
import numpy as np
import matplotlib.pyplot as plt

from sklearn import cluster, metrics, decomposition

# Using pandas library to load csv file into a DataFrame object
# I got the dataset from here: https://raw.githubusercontent.com/vincentarelbundock/Rdatasets/master/csv/datasets/iris.csv
ourData = pd.read_csv("iris.csv")

# Created object `ourData` contains a funny and famous data set of flowers.
# At first, we need to explore, what is in this data set.
# For this, pandas package provides several very useful functions


# The first useful command is `head()`
# It returns first several lines of a dataframe. Very useful to get an idea of what kind of data you have!
print(ourData.head())
#    Unnamed: 0  Sepal.Length  Sepal.Width  Petal.Length  Petal.Width Species
# 0           1           5.1          3.5           1.4          0.2  setosa
# 1           2           4.9          3.0           1.4          0.2  setosa
# 2           3           4.7          3.2           1.3          0.2  setosa
# 3           4           4.6          3.1           1.5          0.2  setosa
# 4           5           5.0          3.6           1.4          0.2  setosa

# Second command, `info()`, can give a more precise information on what data types we operate on
print(ourData.info())
# <class 'pandas.core.frame.DataFrame'>
# Int64Index: 150 entries, 0 to 149
# Data columns (total 6 columns):
# Unnamed: 0      150 non-null int64
# Sepal.Length    150 non-null float64
# Sepal.Width     150 non-null float64
# Petal.Length    150 non-null float64
# Petal.Width     150 non-null float64
# Species         150 non-null object
# dtypes: float64(4), int64(1), object(1)
# memory usage: 8.2+ KB

# The last command is more advanced: `describe()`
# This command gives some statistical information about numeric values in your dataset.
# It is useful to understand what is the range of your values.
print(ourData.describe())
#        Unnamed: 0  Sepal.Length  Sepal.Width  Petal.Length  Petal.Width
# count  150.000000    150.000000   150.000000    150.000000   150.000000
# mean    75.500000      5.843333     3.057333      3.758000     1.199333
# std     43.445368      0.828066     0.435866      1.765298     0.762238
# min      1.000000      4.300000     2.000000      1.000000     0.100000
# 25%     38.250000      5.100000     2.800000      1.600000     0.300000
# 50%     75.500000      5.800000     3.000000      4.350000     1.300000
# 75%    112.750000      6.400000     3.300000      5.100000     1.800000
# max    150.000000      7.900000     4.400000      6.900000     2.500000


# But what about our text column `Species`?
# let see what kinds of values it has?
species = ourData.Species.unique()

print(species)
# ['setosa' 'versicolor' 'virginica']
# Apparently, we have three difference types of flowers in this dataset.


# plot different species in different colours
colors = ['g', 'r', 'b', 'c', 'm', 'k']
species_dict = dict(zip(species, colors))
plt.scatter(ourData['Sepal.Length'], ourData['Sepal.Width'], c=ourData['Species'].apply(lambda x: species_dict[x]))
plt.savefig("real-labels.png")

# Here is our data, but without species information
ourDataNoLabels = pd.DataFrame(ourData, columns=['Sepal.Length', 'Sepal.Width', 'Petal.Length', 'Petal.Width'])

# Now we can try our clustering algorithms!
# We know that there must be 3 different clusters - species of flowers.
# So let's give this hint to the algorithm
kmeans = cluster.KMeans(3).fit(np.array(ourDataNoLabels))
foundLabels = pd.DataFrame(kmeans.labels_, columns=['K-means clusters'])
#print(foundLabels)

plt.scatter(ourData['Sepal.Length'], ourData['Sepal.Width'], c=foundLabels['K-means clusters'].apply(lambda x: colors[x]))
plt.savefig("predicted-labels.png")

mat = pd.crosstab(ourData['Species'], foundLabels['K-means clusters'])
print(mat)
# K-means clusters   0   1   2
# Species
# setosa             0  50   0
# versicolor        48   0   2
# virginica         14   0  36




dbscan = cluster.DBSCAN().fit(np.array(ourDataNoLabels))
foundLabels['DBSCAN clusters'] = pd.DataFrame(dbscan.labels_, columns=['DBSCAN clusters'])
plt.scatter(ourData['Sepal.Length'], ourData['Sepal.Width'], c=foundLabels['DBSCAN clusters'].apply(lambda x: colors[x]))
plt.savefig("predicted-labels2.png")
mat = pd.crosstab(ourData['Species'], foundLabels['DBSCAN clusters'])
print(mat)


#print(metrics.accuracy_score(ourData['Species'].apply(lambda x: species_dict[x]), foundLabels['K-means clusters'].apply(lambda x: colors[x])))


# Run Principal Component Analysis to get more understanding how our data looks like
ourDataReduced = decomposition.PCA(n_components=3).fit_transform(ourDataNoLabels)
plt.scatter(ourDataReduced[:,0], ourDataReduced[:,1], c=ourData['Species'].apply(lambda x: species_dict[x]))
plt.savefig("real-pca-1-2.png")
plt.scatter(ourDataReduced[:,0], ourDataReduced[:,2], c=ourData['Species'].apply(lambda x: species_dict[x]))
plt.savefig("real-pca-1-3.png")
plt.scatter(ourDataReduced[:,1], ourDataReduced[:,2], c=ourData['Species'].apply(lambda x: species_dict[x]))
plt.savefig("real-pca-2-3.png")
